@@ -1,12 +1,13 @@
 // Queue implementation -*- C++ -*-
 
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
+// 2010, 2011
 // Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2, or (at your option)
+// Free Software Foundation; either version 3, or (at your option)
 // any later version.
 
 // This library is distributed in the hope that it will be useful,
@@ -14,19 +15,14 @@
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 
-// You should have received a copy of the GNU General Public License along
-// with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-// USA.
+// Under Section 7 of GPL version 3, you are granted additional
+// permissions described in the GCC Runtime Library Exception, version
+// 3.1, as published by the Free Software Foundation.
 
-// As a special exception, you may use this file as part of a free software
-// library without restriction.  Specifically, if other files instantiate
-// templates or use macros or inline functions from this file, or you compile
-// this file and link it with other files to produce an executable, this
-// file does not by itself cause the resulting executable to be covered by
-// the GNU General Public License.  This exception does not however
-// invalidate any other reasons why the executable file might be covered by
-// the GNU General Public License.
+// You should have received a copy of the GNU General Public License and
+// a copy of the GCC Runtime Library Exception along with this program;
+// see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see
+// <http://www.gnu.org/licenses/>.
 
 /*
  *
@@ -54,9 +50,9 @@
  * purpose.  It is provided "as is" without express or implied warranty.
  */
 
-/** @file stl_queue.h
+/** @file bits/stl_queue.h
  *  This is an internal header file, included by other library headers.
- *  You should not attempt to use it directly.
+ *  Do not attempt to use it directly. @headername{queue}
  */
 
 #ifndef _STL_QUEUE_H
@@ -65,13 +61,14 @@
 #include <bits/concept_check.h>
 #include <debug/debug.h>
 
-_GLIBCXX_BEGIN_NAMESPACE(std)
+namespace std _GLIBCXX_VISIBILITY(default)
+{
+_GLIBCXX_BEGIN_NAMESPACE_VERSION
 
   /**
    *  @brief  A standard container giving FIFO behavior.
    *
-   *  @ingroup Containers
-   *  @ingroup Sequences
+   *  @ingroup sequences
    *
    *  Meets many of the requirements of a
    *  <a href="tables.html#65">container</a>,
@@ -87,7 +84,7 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
    *  that supports @c front, @c back, @c push_back, and @c pop_front,
    *  such as std::list or an appropriate user-defined type.
    *
-   *  Members not found in "normal" containers are @c container_type,
+   *  Members not found in @a normal containers are @c container_type,
    *  which is a typedef for the second Sequence parameter, and @c push and
    *  @c pop, which are standard %queue/FIFO operations.
   */
@@ -143,16 +140,6 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
       explicit
       queue(_Sequence&& __c = _Sequence())
       : c(std::move(__c)) { }
-
-      queue(queue&& __q)
-      : c(std::move(__q.c)) { }
-
-      queue&
-      operator=(queue&& __q)
-      {
-	c = std::move(__q.c);
-	return *this;
-      }
 #endif
 
       /**
@@ -220,16 +207,19 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
        *  to it.  The time complexity of the operation depends on the
        *  underlying sequence.
        */
-#ifndef __GXX_EXPERIMENTAL_CXX0X__
       void
       push(const value_type& __x)
       { c.push_back(__x); }
-#else
-      // NB: DR 756.
+
+#ifdef __GXX_EXPERIMENTAL_CXX0X__
+      void
+      push(value_type&& __x)
+      { c.push_back(std::move(__x)); }
+
       template<typename... _Args>
         void
-        push(_Args&&... __args)
-	{ c.push_back(std::forward<_Args>(__args)...); }
+        emplace(_Args&&... __args)
+	{ c.emplace_back(std::forward<_Args>(__args)...); }
 #endif
 
       /**
@@ -252,8 +242,11 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
 
 #ifdef __GXX_EXPERIMENTAL_CXX0X__
       void
-      swap(queue&& __q)
-      { c.swap(__q.c); }
+      swap(queue& __q)
+      {
+	using std::swap;
+	swap(c, __q.c);
+      }
 #endif
     };
 
@@ -321,22 +314,15 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
     swap(queue<_Tp, _Seq>& __x, queue<_Tp, _Seq>& __y)
     { __x.swap(__y); }
 
-  template<typename _Tp, typename _Seq>
-    inline void
-    swap(queue<_Tp, _Seq>&& __x, queue<_Tp, _Seq>& __y)
-    { __x.swap(__y); }
-
-  template<typename _Tp, typename _Seq>
-    inline void
-    swap(queue<_Tp, _Seq>& __x, queue<_Tp, _Seq>&& __y)
-    { __x.swap(__y); }
+  template<typename _Tp, typename _Seq, typename _Alloc>
+    struct uses_allocator<queue<_Tp, _Seq>, _Alloc>
+    : public uses_allocator<_Seq, _Alloc>::type { };
 #endif
 
   /**
    *  @brief  A standard container automatically sorting its contents.
    *
-   *  @ingroup Containers
-   *  @ingroup Sequences
+   *  @ingroup sequences
    *
    *  This is not a true container, but an @e adaptor.  It holds
    *  another container, and provides a wrapper interface to that
@@ -354,7 +340,7 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
    *  priority comparisons.  It defaults to @c less<value_type> but
    *  can be anything defining a strict weak ordering.
    *
-   *  Members not found in "normal" containers are @c container_type,
+   *  Members not found in @a normal containers are @c container_type,
    *  which is a typedef for the second Sequence parameter, and @c
    *  push, @c pop, and @c top, which are standard %queue operations.
    *
@@ -429,7 +415,7 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
        *  the copy according to @a x.
        *
        *  For more information on function objects, see the
-       *  documentation on @link s20_3_1_base functor base
+       *  documentation on @link functors functor base
        *  classes@endlink.
        */
 #ifndef __GXX_EXPERIMENTAL_CXX0X__
@@ -465,17 +451,6 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
 	  c.insert(c.end(), __first, __last);
 	  std::make_heap(c.begin(), c.end(), comp);
 	}
-
-      priority_queue(priority_queue&& __pq)
-      : c(std::move(__pq.c)), comp(std::move(__pq.comp)) { }
-
-      priority_queue&
-      operator=(priority_queue&& __pq)
-      {
-	c = std::move(__pq.c);
-	comp = std::move(__pq.comp);
-	return *this;
-      }
 #endif
 
       /**
@@ -509,20 +484,26 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
        *  The time complexity of the operation depends on the underlying
        *  sequence.
        */
-#ifndef __GXX_EXPERIMENTAL_CXX0X__
       void
       push(const value_type& __x)
       {
 	c.push_back(__x);
 	std::push_heap(c.begin(), c.end(), comp);
       }
-#else
-      // NB: DR 756.
+
+#ifdef __GXX_EXPERIMENTAL_CXX0X__
+      void
+      push(value_type&& __x)
+      {
+	c.push_back(std::move(__x));
+	std::push_heap(c.begin(), c.end(), comp);
+      }
+
       template<typename... _Args>
         void
-        push(_Args&&... __args)
-	{ 
-	  c.push_back(std::forward<_Args>(__args)...);
+        emplace(_Args&&... __args)
+	{
+	  c.emplace_back(std::forward<_Args>(__args)...);
 	  std::push_heap(c.begin(), c.end(), comp);
 	}
 #endif
@@ -548,10 +529,10 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
 
 #ifdef __GXX_EXPERIMENTAL_CXX0X__
       void
-      swap(priority_queue&& __pq)
+      swap(priority_queue& __pq)
       {
 	using std::swap;
-	c.swap(__pq.c);
+	swap(c, __pq.c);
 	swap(comp, __pq.comp);
       }
 #endif
@@ -566,19 +547,13 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
 	 priority_queue<_Tp, _Sequence, _Compare>& __y)
     { __x.swap(__y); }
 
-  template<typename _Tp, typename _Sequence, typename _Compare>
-    inline void
-    swap(priority_queue<_Tp, _Sequence, _Compare>&& __x,
-	 priority_queue<_Tp, _Sequence, _Compare>& __y)
-    { __x.swap(__y); }
-
-  template<typename _Tp, typename _Sequence, typename _Compare>
-    inline void
-    swap(priority_queue<_Tp, _Sequence, _Compare>& __x,
-	 priority_queue<_Tp, _Sequence, _Compare>&& __y)
-    { __x.swap(__y); }
+  template<typename _Tp, typename _Sequence, typename _Compare,
+	   typename _Alloc>
+    struct uses_allocator<priority_queue<_Tp, _Sequence, _Compare>, _Alloc>
+    : public uses_allocator<_Sequence, _Alloc>::type { };
 #endif
 
-_GLIBCXX_END_NAMESPACE
+_GLIBCXX_END_NAMESPACE_VERSION
+} // namespace
 
 #endif /* _STL_QUEUE_H */
