@@ -61,12 +61,10 @@ void test01()
   locale loc(locale::classic(), cvt);
 
   wfilebuf fb;
-  fb.pubsetbuf(0, 0);
   fb.pubimbue(loc);
 
   fb.open(name, ios_base::out);
   fb.sputc(L'a');
-  fb.in_avail(); // showmanyc() should have no effect on close().
 
   VERIFY( !cvt->unshift_called );
   fb.close();
