@@ -1,11 +1,12 @@
 // 1999-04-12 bkoz
 
-// Copyright (C) 1999, 2000, 2002, 2003 Free Software Foundation, Inc.
+// Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009
+// Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2, or (at your option)
+// Free Software Foundation; either version 3, or (at your option)
 // any later version.
 
 // This library is distributed in the hope that it will be useful,
@@ -14,15 +15,15 @@
 // GNU General Public License for more details.
 
 // You should have received a copy of the GNU General Public License along
-// with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-// USA.
+// with this library; see the file COPYING3.  If not see
+// <http://www.gnu.org/licenses/>.
 
 // 27.6.1.2.2 arithmetic extractors
 
 #include <istream>
 #include <sstream>
 #include <locale>
+#include <limits>
 #include <testsuite_hooks.h>
 
 // libstdc++/3720 part two
@@ -57,8 +58,8 @@ void test13()
   i = 0;
   iss2.str(digits);
   iss2.clear();
-  iss2 >> i; 
-  VERIFY( i == 0 );
+  iss2 >> i;
+  VERIFY( i == numeric_limits<int>::max() );
   VERIFY( iss2.fail() );
 }
 
