@@ -1,6 +1,7 @@
 // 1999-10-11 bkoz
 
-// Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009
+// Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
+// 2007, 2009, 2010
 // Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
@@ -40,12 +41,12 @@ public:
   check_pointers()
   { 
     bool test __attribute__((unused)) = true;
-    VERIFY( this->eback() == NULL );
-    VERIFY( this->gptr() == NULL );
-    VERIFY( this->egptr() == NULL );
-    VERIFY( this->pbase() == NULL );
-    VERIFY( this->pptr() == NULL );
-    VERIFY( this->epptr() == NULL );
+    VERIFY( !this->eback() );
+    VERIFY( !this->gptr() );
+    VERIFY( !this->egptr() );
+    VERIFY( !this->pbase() );
+    VERIFY( !this->pptr() );
+    VERIFY( !this->epptr() );
     return test;
   }
 
