@@ -1,7 +1,7 @@
 // { dg-options "-std=gnu++0x" }
 // { dg-do compile }
 
-// Copyright (C) 2008, 2009 Free Software Foundation, Inc.
+// Copyright (C) 2008, 2009, 2010 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -18,10 +18,10 @@
 // with this library; see the file COPYING3.  If not see
 // <http://www.gnu.org/licenses/>.
 
-#include <stdatomic.h>
+#include <atomic>
 
 void test01()
 {
   using namespace std;
-  atomic_flag af = ATOMIC_FLAG_INIT;
+  atomic_flag af __attribute__((unused)) = ATOMIC_FLAG_INIT;
 }
